from je_editor.utils.editor_content.editor_data import editor_data_dict


def set_language(exec_manager, language):
    """
    :param exec_manager: which exec manager change program language
    :param language: set exec manager program language
    """
    if type(language) is list:
        exec_manager.program_language = language[0]
    else:
        exec_manager.program_language = language
    editor_data_dict["language"] = language



from tkinter import Menu

from je_editor.ui.ui_event.clear_result.clear_result import clear_result_area
from je_editor.ui.ui_event.execute.execute_code.exec_code import stop_program
from je_editor.ui.ui_event.execute.execute_shell_command.run_on_shell import execute_shell_command


def build_run_menu(editor_instance):
    # Run menu
    editor_instance.run_menu = Menu(editor_instance.menu, tearoff=0)
    editor_instance.run_menu.add_command(
        label="Run Program",
        command=editor_instance.ui_execute_program
    )
    editor_instance.run_menu.add_command(
        label="Run on shell",
        command=lambda: execute_shell_command(
            editor_instance.program_run_result_textarea,
            editor_instance.code_editor_textarea
        )
    )
    editor_instance.run_menu.add_command(
        label="Stop Program",
        command=lambda: stop_program(editor_instance.exec_manager)
    )
    editor_instance.run_menu.add_command(
        label="Clean Result", command=lambda: clear_result_area(
            editor_instance.program_run_result_textarea
        )
    )



import os.path
import queue
import shutil
import subprocess
from pathlib import Path
from threading import Thread
from tkinter import DISABLED
from tkinter import END
from tkinter import NORMAL

from je_editor.utils.exception.exception_tags import file_not_fond_error
from je_editor.utils.exception.exception_tags import compiler_not_found_error
from je_editor.utils.exception.exceptions import JEditorExecException

from je_editor.utils.language_data_module.language_compiler_data_module import language_compiler
from je_editor.utils.language_data_module.language_param_data_module import language_compiler_param


class ExecManager(object):

    def __init__(
            self,
            program_run_result_textarea,
            process_error_function,
            main_window,
            running_menu,
            program_language="python",
            program_encoding="utf-8"
    ):
        """
        :param program_run_result_textarea:  program run result textarea
        :param process_error_function: when process error call this function
        :param main_window: tkinter main window
        :param running_menu: menu when running change state
        :param program_language: which program language
        :param program_encoding: which encoding
        """
        self.read_program_error_output_from_thread = None
        self.read_program_output_from_thread = None
        self.main_window = main_window
        self.still_run_program = True
        self.run_program_result_textarea = program_run_result_textarea
        self.process_error_function = process_error_function
        self.process = None
        self.run_output_queue = queue.Queue()
        self.run_error_queue = queue.Queue()
        self.program_language = program_language
        self.running_menu = running_menu
        self.program_encoding = program_encoding

    def exec_code(self, exec_file_name):
        """
        :param exec_file_name: string file will open to run
        :return: if error return result and True else return result and False
        """
        try:
            self.exit_program()
            self.running_menu.entryconfigure(1, label="Rerun")
            self.running_menu.entryconfigure(3, state="normal")
            self.run_program_result_textarea.configure(state=NORMAL)
            self.run_program_result_textarea.delete("1.0", "end-1c")
            self.run_program_result_textarea.configure(state=DISABLED)
            reformat_os_file_path = os.path.abspath(exec_file_name)
            # detect file is exist
            try:
                if not Path(exec_file_name).exists():
                    raise JEditorExecException(file_not_fond_error)
            except OSError as error:
                raise JEditorExecException(error)
            compiler_path = shutil.which(self.program_language)
            if compiler_path is None and self.program_language == "python":
                compiler_path = shutil.which("python3")
            if compiler_path is None:
                raise JEditorExecException(compiler_not_found_error)
            exec_file = reformat_os_file_path
            # precompile
            if self.program_language in language_compiler:
                self.process = subprocess.Popen(
                    [
                        shutil.which(language_compiler.get(self.program_language)),
                        language_compiler_param.get(self.program_language),
                        reformat_os_file_path
                    ],
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE,
                )
                while self.process.returncode is None:
                    self.process.poll()
            # run program
            execute_program_list = [compiler_path, exec_file]
            self.process = subprocess.Popen(
                execute_program_list,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                shell=False
            )
            self.still_run_program = True
            # program output message queue thread
            self.read_program_output_from_thread = Thread(
                target=self.read_program_output_from_process,
                daemon=True
            ).start()
            # program error message queue thread
            self.read_program_error_output_from_thread = Thread(
                target=self.read_program_error_output_from_process,
                daemon=True
            ).start()
            # show which file execute
            self.run_program_result_textarea.configure(state=NORMAL)
            self.run_program_result_textarea.insert(END, compiler_path + " " + reformat_os_file_path + "\n")
            self.run_program_result_textarea.configure(state=DISABLED)
            # start ui update
            self.edit_tkinter_text()
        except Exception as error:
            print(repr(error))

    # ui update method
    def edit_tkinter_text(self):
        try:
            self.run_program_result_textarea.configure(state=NORMAL)
            if not self.run_error_queue.empty():
                error_message = self.run_error_queue.get_nowait()
                self.process_error_function(self.run_program_result_textarea, error_message)
            if not self.run_output_queue.empty():
                output_message = self.run_output_queue.get_nowait()
                self.run_program_result_textarea.insert(END, output_message)
            self.run_program_result_textarea.configure(state=DISABLED)
        except queue.Empty:
            pass
        if self.process.returncode == 0:
            self.exit_program()
        elif self.process.returncode is not None:
            self.exit_program()
        if self.still_run_program:
            self.main_window.after(1, self.edit_tkinter_text)
            # poll return code
            self.process.poll()
        else:
            self.running_menu.entryconfigure(1, label="Run")
            self.running_menu.entryconfigure(3, state="disable")

    # exit program change run flag to false and clean read thread and queue and process
    def exit_program(self):
        self.still_run_program = False
        if self.read_program_output_from_thread is not None:
            self.read_program_output_from_thread = None
        if self.read_program_error_output_from_thread is not None:
            self.read_program_error_output_from_thread = None
        self.print_and_clear_queue()
        if self.process is not None:
            self.process.terminate()

    def print_and_clear_queue(self):
        try:
            self.run_program_result_textarea.configure(state=NORMAL)
            for std_output in iter(self.run_output_queue.get_nowait, None):
                self.run_program_result_textarea.insert(END, std_output + "\n")
            for std_err in iter(self.run_error_queue.get_nowait, None):
                self.run_program_result_textarea.insert(END, std_err, "warning", "\n")
            self.run_program_result_textarea.configure(state=DISABLED)
        except queue.Empty:
            pass
        self.run_output_queue = queue.Queue()
        self.run_error_queue = queue.Queue()

    def read_program_output_from_process(self):
        while self.still_run_program:
            program_output_data = self.process.stdout.raw.read(1024000).decode(self.program_encoding)
            self.run_output_queue.put_nowait(program_output_data)

    def read_program_error_output_from_process(self):
        while self.still_run_program:
            program_error_output_data = self.process.stderr.raw.read(1024000).decode(self.program_encoding)
            self.run_error_queue.put_nowait(program_error_output_data)



import json
import os
from pathlib import Path
from threading import Lock

from je_editor.utils.editor_content.editor_data import editor_data_dict
from je_editor.utils.exception.exceptions import JEditorContentFileException
from je_editor.utils.json_format.json_process import reformat_json


def read_output_content():
    """
    read the editor content
    """
    lock = Lock()
    try:
        lock.acquire()
        cwd = os.getcwd()
        file_path = Path(cwd + "/je_editor_content.json")
        if file_path.exists() and file_path.is_file():
            with open(cwd + "/je_editor_content.json", "r+") as read_file:
                return read_file.read()
    except JEditorContentFileException:
        raise JEditorContentFileException
    finally:
        lock.release()


def write_output_content():
    """
    write the editor content
    """
    lock = Lock()
    try:
        lock.acquire()
        cwd = os.getcwd()
        with open(cwd + "/je_editor_content.json", "w+") as file_to_write:
            file_to_write.write(reformat_json(json.dumps(editor_data_dict)))
    except JEditorContentFileException:
        raise JEditorContentFileException
    finally:
        lock.release()


def save_content_and_quit():
    """
    set content data and write
    """
    write_output_content()


def open_content_and_start():
    """
    read data and set content
    """
    temp_content = read_output_content()
    if temp_content is not None:
        return json.loads(temp_content)



from tkinter import ttk, Text, Tk


def tkinter_ui_setting(editor_instance, main_window: Tk):
    # set main window title and add main frame
    editor_instance.main_window = main_window
    editor_instance.main_window.title("je_editor")
    editor_instance.code_edit_frame = ttk.Frame(editor_instance.main_window, padding="3 3 12 12")
    editor_instance.program_run_result_frame = ttk.Frame(editor_instance.main_window, padding="3 3 12 12")
    # set code edit
    editor_instance.code_editor_textarea = Text(
        editor_instance.code_edit_frame, undo=True, autoseparators=True, maxundo=-1, wrap="none")
    editor_instance.code_editor_textarea.configure(state="normal")
    editor_instance.code_editor_textarea.config(tabs="1c")
    editor_instance.code_editor_textarea_scrollbar_y = ttk.Scrollbar(
        editor_instance.code_edit_frame, orient="vertical",
        command=editor_instance.code_editor_textarea.yview
    )
    editor_instance.code_editor_textarea_scrollbar_x = ttk.Scrollbar(
        editor_instance.code_edit_frame, orient="horizontal",
        command=editor_instance.code_editor_textarea.xview)
    editor_instance.code_editor_textarea["yscrollcommand"] = editor_instance.code_editor_textarea_scrollbar_y.set
    editor_instance.code_editor_textarea["xscrollcommand"] = editor_instance.code_editor_textarea_scrollbar_x.set
    # run result
    editor_instance.program_run_result_textarea = Text(editor_instance.program_run_result_frame, wrap="none")
    editor_instance.program_run_result_textarea_scrollbar_y = ttk.Scrollbar(
        editor_instance.program_run_result_frame, orient="vertical",
        command=editor_instance.program_run_result_textarea.yview)
    editor_instance.program_run_result_textarea_scrollbar_x = ttk.Scrollbar(
        editor_instance.program_run_result_frame, orient="horizontal",
        command=editor_instance.program_run_result_textarea.xview)
    editor_instance.program_run_result_textarea[
        "yscrollcommand"] = editor_instance.program_run_result_textarea_scrollbar_y.set
    editor_instance.program_run_result_textarea[
        "xscrollcommand"] = editor_instance.program_run_result_textarea_scrollbar_x.set
    editor_instance.program_run_result_textarea.configure(state="disabled")



from je_editor.utils.file.save.save_file import SaveThread


def start_auto_save(auto_save_thread_object, current_file, code_editor):
    """
    :param auto_save_thread_object: auto save thread object
    :param current_file: current edit file
    :param code_editor: code editor we load text to save
    :return: new auto save thread object
    """
    if auto_save_thread_object is not None and current_file is not None:
        auto_save_thread_object.file = current_file
        return auto_save_thread_object
    elif current_file is not None and auto_save_thread_object is None:
        auto_save = SaveThread(current_file, code_editor)
        auto_save.start()
        return auto_save



from tkinter import NORMAL
from tkinter import DISABLED

from je_editor.ui.ui_event.text_process.shell.shell_text import run_on_shell
from je_editor.ui.ui_event.text_process.program_exec.process_error import process_error_text


def execute_shell_command(program_run_result_textarea, code_editor):
    """
    :param program_run_result_textarea tkinter textarea to show result
    :param code_editor get the command to run
    """
    program_run_result_textarea.configure(state=NORMAL)
    program_run_result_textarea.delete("1.0", "end-1c")
    temp_result = run_on_shell(code_editor.get("1.0", "end-1c"))
    if temp_result[1]:
        process_error_text(program_run_result_textarea, temp_result[0])
    else:
        program_run_result_textarea.insert("1.0", temp_result[0])
    program_run_result_textarea.configure(state=DISABLED)



from je_editor.utils.file.open.open_file import open_file


def open_file_to_read(code_editor):
    """
    :param code_editor the editor to insert file content
    :return readied file
    show open file dialog
    if choose some file
        open and read it insert content to tkinter code_editor
        change current file
        start auto save
    """
    temp_to_check_file = open_file()
    if temp_to_check_file is not None and temp_to_check_file != "":
        code_editor.delete("1.0", "end-1c")
        code_editor.insert("end-1c", temp_to_check_file[1])
        return temp_to_check_file[0]
    else:
        return None



from tkinter import NORMAL, DISABLED


def clear_result_area(result_area_to_clear=None):
    if result_area_to_clear is not None:
        result_area_to_clear.configure(state=NORMAL)
        result_area_to_clear.delete("1.0", "end-1c")
        result_area_to_clear.configure(state=DISABLED)


from tkinter import font
from tkinter.font import Font


def get_font(root, **kwargs):
    """
    :param root: get current font families
    :param kwargs: another param
    """
    return font.families(root, **kwargs)


def create_new_font(font_family: str, font_size: int = 12, **kwargs):
    """
    :param font_family: which font family choose to create new font
    :param font_size: font size
    :param kwargs:  another param
    """
    new_font = Font(font=(font_family, font_size), **kwargs)
    return new_font



from tkinter import IntVar

from je_editor.utils.keyword.keyword_list import keyword_list
from je_editor.utils.theme.theme import theme_dict


class HighlightText(object):

    def __init__(self, tkinter_text, start_position="1.0", end_position="end"):
        """
        :param tkinter_text: want to set highlight's tkinter text
        :param start_position: search start position
        :param end_position: search end position
        """
        self.tkinter_text = tkinter_text
        self.start_position = start_position
        self.end_position = end_position
        # theme dict on theme
        self.theme = theme_dict
        # use regexp
        self.tkinter_text.regexp = True
        # bind to keyboard key release
        self.tkinter_text.bind("<KeyRelease>", self.search)

    def search(self, event=None):
        """
        :param event: tkinter event
        create temp var tag
        remove tag
        search all word in keyword_list and tag
        """
        tag = "temp"
        for tag in self.tkinter_text.tag_names():
            self.tkinter_text.tag_remove(tag, self.start_position, self.end_position)
        count_var = IntVar()
        for word in keyword_list:
            position = '1.0'
            self.tkinter_text.tag_config(word, foreground=self.theme.get("tag_keyword_color"))
            while self.tkinter_text.compare(position, "<", "end"):
                find_function_index = self.tkinter_text.search(
                    "\m" + word + "\M",
                    position, self.end_position,
                    count=count_var,
                    regexp=True
                )
                if not find_function_index:
                    break
                position = '{}+{}c'.format(find_function_index, len(word))
                self.tkinter_text.tag_add(tag, find_function_index, position)



def build_menu(editor_instance):
    # add and config
    editor_instance.menu.add_cascade(label="File", menu=editor_instance.file_menu)
    editor_instance.menu.add_cascade(label="Run", menu=editor_instance.run_menu)
    editor_instance.main_window.config(menu=editor_instance.menu)



from je_editor.utils.editor_content.content_save import save_content_and_quit


def close_event(tkinter_window, exec_manager):
    """
    :param exec_manager:
    :param tkinter_window: the tkinter window will close
    :return: no return value
    """
    save_content_and_quit()
    exec_manager.exit_program()
    tkinter_window.destroy()



import sys

from je_editor.ui.ui_event.change_font.change_font import change_font, change_font_size
from je_editor.ui.ui_event.encoding.set_encoding import set_encoding
from je_editor.ui.ui_event.language.set_language import set_language
from je_editor.ui.ui_event.tag_keyword.tag_keyword import HighlightText
from je_editor.utils.editor_content.content_save import open_content_and_start
from je_editor.utils.editor_content.editor_data import editor_data_dict
from je_editor.utils.exception.exceptions import JEditorContentFileException
from je_editor.utils.language.language_data_module import language_list
from je_editor.utils.language_data_module.language_compiler_data_module import language_compiler
from je_editor.utils.language_data_module.language_param_data_module import language_compiler_param


def content_init(editor_instance):
    # Highlight word
    editor_instance.highlight_text = HighlightText(editor_instance.code_editor_textarea)
    # file from output content
    editor_instance.file_from_output_content = open_content_and_start()
    try:
        if editor_instance.file_from_output_content is not None:
            editor_instance.current_file = editor_instance.ui_open_last_edit_file()
            editor_instance.highlight_text.search()
            if editor_instance.file_from_output_content.get("theme") is not None:
                editor_instance.highlight_text.theme = editor_instance.file_from_output_content.get("theme")
                editor_data_dict["theme"] = editor_instance.file_from_output_content.get(
                    "theme")
            if editor_instance.file_from_output_content.get("language") is not None:
                language_from_content = editor_instance.file_from_output_content.get("language")
                for language in language_from_content:
                    if language not in language_list:
                        language_list.append(language)
                set_language(editor_instance.exec_manager, language_from_content)
            if editor_instance.file_from_output_content.get("encoding") is not None:
                set_encoding(editor_instance.exec_manager, editor_instance.file_from_output_content.get("encoding"))
            if editor_instance.file_from_output_content.get("font") is not None:
                change_font(
                    editor_instance.code_editor_textarea,
                    editor_instance.program_run_result_textarea,
                    editor_instance.file_from_output_content.get("font")
                )
            if editor_instance.file_from_output_content.get("font_size") is not None:
                change_font_size(
                    editor_instance.code_editor_textarea,
                    editor_instance.program_run_result_textarea,
                    editor_instance.file_from_output_content.get("font_size")
                )
            try:
                if editor_instance.file_from_output_content.get("language_precompiler") is not None:
                    language_compiler.update(editor_instance.file_from_output_content.get("language_precompiler"))
                    editor_data_dict["language_precompiler"] = editor_instance.file_from_output_content.get(
                        "language_precompiler")
                if editor_instance.file_from_output_content.get("language_compiler_param") is not None:
                    language_compiler_param.update(editor_instance.file_from_output_content.get("language_compiler_param"))
                    editor_data_dict["language_compiler_param"] = editor_instance.file_from_output_content.get(
                        "language_compiler_param")
            except JEditorContentFileException as error:
                print(repr(error), file=sys.stderr)
            if editor_instance.file_from_output_content.get("tab_size") is not None:
                editor_data_dict["tab_size"] = editor_instance.file_from_output_content.get(
                    "tab_size")
                editor_instance.code_editor_textarea.config(tabs=editor_instance.file_from_output_content.get("tab_size"))
    except JEditorContentFileException as error:
        print(repr(error), file=sys.stderr)
    editor_instance.highlight_text.search()



from je_editor.utils.theme.theme import theme_dict


"""
if you are using python3
language_precompiler and language_compiler_param value None is right
if you want to add param you can edit language_compiler_param
"""

editor_data_dict = {
    "last_file": None,
    "theme": theme_dict,
    "language": ["python3"],
    "language_precompiler": None,
    "language_compiler_param": None,
    "encoding": "utf-8",
    "font": "TkDefaultFont",
    "font_size": 12,
    "tab_size": "1c",
}



from tkinter import END


def process_error_text(tkinter_text, error_text):
    """
    :param tkinter_text: tkinter textarea widget
    :param error_text: error text insert in to tkinter_text
    """
    tkinter_text.tag_configure("warning", foreground="red")
    tkinter_text.insert(END, error_text, "warning", "\n")



from je_editor.utils.editor_content.editor_data import editor_data_dict


def set_encoding(exec_manager, encoding):
    """
    :param exec_manager: which program exec manage change encoding
    :param encoding: which encoding choose
    """
    exec_manager.program_encoding = encoding
    editor_data_dict["encoding"] = encoding



import os
from tkinter import Menu
from tkinter import Tk
from tkinter import ttk

from je_editor.ui.editor_main_ui.content_init.content_init import content_init
from je_editor.ui.editor_main_ui.menu.build_menu import build_menu
from je_editor.ui.editor_main_ui.menu.file_menu.build_file_menu import build_file_menu
from je_editor.ui.editor_main_ui.menu.popup_menu.build_popup_menu import build_popup_menu
from je_editor.ui.editor_main_ui.menu.run_menu.build_run_menu import build_run_menu
from je_editor.ui.editor_main_ui.protocal_setting.set_ui_protocol import tkinter_set_protocol
from je_editor.ui.editor_main_ui.ui_event_bind.event_bind import tkinter_event_bind
from je_editor.ui.editor_main_ui.ui_grid.build_ui_grid import build_grid
from je_editor.ui.editor_main_ui.ui_setting.ui_setting import tkinter_ui_setting
from je_editor.ui.ui_event.auto_save.start_auto_save.start_auto_save import start_auto_save
from je_editor.ui.ui_event.close.close_event import close_event
from je_editor.ui.ui_event.execute.execute_code.exec_code import execute_code
from je_editor.ui.ui_event.open_file.open_file_to_read.open_file_to_read import open_file_to_read
from je_editor.ui.ui_event.open_file.open_last_edit_file.open_last_edit_file import open_last_edit_file
from je_editor.ui.ui_event.save_file.save_file_to_open.save_file_to_open import save_file_then_can_run
from je_editor.ui.ui_event.save_file.save_file_to_open.save_file_to_open import save_file_to_open
from je_editor.ui.ui_event.text_process.program_exec.exec_text import ExecManager
from je_editor.ui.ui_event.text_process.program_exec.process_error import process_error_text
from je_editor.utils.editor_content.editor_data import editor_data_dict


class EditorMain(object):

    # start editor and start auto save if auto save not start
    def start_editor(self):
        self.auto_save_thread = start_auto_save(self.auto_save_thread, self.current_file, self.code_editor_textarea)
        if self.debug_run:
            self.close_event()
        else:
            self.main_window.mainloop()

    # editor close event
    def close_event(self):
        editor_data_dict["last_file"] = self.current_file
        if self.file_from_output_content is not None:
            self.file_from_output_content["last_file"] = self.current_file
        close_event(self.main_window, self.exec_manager)

    # editor open file from path
    def ui_open_file_to_read(self, event=None):
        temp = open_file_to_read(self.code_editor_textarea)
        self.current_file = temp
        self.highlight_text.search()
        self.auto_save_thread = start_auto_save(self.auto_save_thread, self.current_file, self.code_editor_textarea)

    # save editor file to choose path
    def ui_save_file_to_open(self, event=None):
        self.current_file = save_file_to_open(self.code_editor_textarea)
        self.auto_save_thread = start_auto_save(self.auto_save_thread, self.current_file, self.code_editor_textarea)

    def ui_open_last_edit_file(self):
        self.highlight_text.search()
        return open_last_edit_file(self.file_from_output_content.get("last_file"), self.code_editor_textarea)

    def ui_execute_program(self, event=None):
        if self.current_file is not None:
            save_file_then_can_run(self.current_file, self.code_editor_textarea)
        execute_code(self.current_file, self.ui_save_file_to_open, self.exec_manager)

    def ui_show_popup_menu(self, event):
        """
        :param event: tkinter event bind Button-3
        """
        self.popup_menu.tk_popup(event.x_root, event.y_root)

    # default event
    def do_test(self, event=None):
        print(self.debug_run)

    def __init__(self, use_theme=None, debug=False, main_window=Tk()):
        """
        :param use_theme: what theme editor used
        :param main_window: Tk instance
        """
        # ui param
        self.popup_menu = None
        self.program_run_result_textarea = None
        self.code_editor_textarea = None
        self.main_window = None
        os.environ["PYTHONUNBUFFERED"] = "1"
        # is this test run?
        self.debug_run = debug
        # Auto save thread
        self.auto_save_thread = None
        # current file
        self.current_file = None
        # style
        self.file_from_output_content = None
        self.highlight_text = None
        self.style = ttk.Style()
        if use_theme is not None:
            self.style.theme_use(use_theme)
        tkinter_ui_setting(self, main_window)
        tkinter_set_protocol(self)
        # Menubar
        # Main menu
        self.menu = Menu(self.main_window)
        if self.current_file is not None:
            self.auto_save_thread = start_auto_save(self.auto_save_thread, self.current_file, self.code_editor_textarea)
        self.exec_manager = ExecManager(
            program_run_result_textarea=self.program_run_result_textarea,
            process_error_function=process_error_text,
            main_window=self.main_window,
            running_menu=self.menu
        )
        # ui init should before grid and menu init
        content_init(self)
        build_grid(self)
        tkinter_event_bind(self)
        # menu add command
        # main menu bar
        build_run_menu(self)
        build_file_menu(self)
        build_popup_menu(self)
        build_menu(self)

    def use_choose_theme(self, use_theme=None):
        self.style.theme_use(use_theme)


def start_editor(use_theme=None, **kwargs):
    new_editor = EditorMain(use_theme=use_theme, **kwargs)
    new_editor.start_editor()
    return new_editor



from je_editor.ui.ui_event.execute.execute_shell_command.run_on_shell import execute_shell_command


def tkinter_event_bind(editor_instance):
    # bind
    editor_instance.program_run_result_textarea.bind(
        "<1>",
        lambda event: editor_instance.program_run_result_textarea.focus_set()
    )
    editor_instance.main_window.bind("<Button-3>", editor_instance.ui_show_popup_menu)
    editor_instance.main_window.bind("<Control-Key-o>", editor_instance.ui_open_file_to_read)
    editor_instance.main_window.bind("<Control-Key-s>", editor_instance.ui_save_file_to_open)
    editor_instance.main_window.bind(
        "<Control-Key-F5>",
        editor_instance.ui_execute_program
    )
    editor_instance.main_window.bind(
        "<Control-Key-F6>",
        lambda bind_exec_shell_command: execute_shell_command(editor_instance.program_run_result_textarea,
                                                              editor_instance.code_editor_textarea)
    )



def build_grid(editor_instance):
    # grid and menu set
    editor_instance.code_edit_frame.grid(column=0, row=0, sticky="nsew")
    editor_instance.program_run_result_frame.grid(column=0, row=1, sticky="nsew")
    editor_instance.main_window.grid_rowconfigure(0, weight=1)
    editor_instance.main_window.grid_rowconfigure(1, weight=1)
    editor_instance.code_editor_textarea.grid(column=0, row=0, sticky="nsew")
    editor_instance.code_editor_textarea_scrollbar_y.grid(column=1, row=0, sticky="ns")
    editor_instance.code_editor_textarea_scrollbar_x.grid(column=0, row=2, sticky="nsew")
    editor_instance.program_run_result_textarea.grid(column=0, row=1, sticky="nsew")
    editor_instance.program_run_result_textarea_scrollbar_y.grid(column=1, row=1, sticky="ns")
    editor_instance.program_run_result_textarea_scrollbar_x.grid(column=0, row=3, sticky="nsew")
    # set resize
    editor_instance.code_edit_frame.columnconfigure(0, weight=1)
    editor_instance.code_edit_frame.rowconfigure(0, weight=1)
    editor_instance.program_run_result_frame.columnconfigure(0, weight=1)
    editor_instance.program_run_result_frame.rowconfigure(1, weight=1)
    editor_instance.main_window.columnconfigure(0, weight=1)
    editor_instance.main_window.rowconfigure(0, weight=1)



def execute_code(current_file, save_function, exec_manager):
    """
    :param exec_manager:
    :param current_file will execute file
    :param save_function if current_file is None or "" save and run
    """
    if current_file is not None and current_file != "":
        exec_manager.exec_code(current_file)
    else:
        save_function()
        if current_file is not None and current_file != "":
            exec_manager.exec_code(current_file)


def stop_program(exec_manager):
    """
    :param exec_manager: call exec manager stop program method to stop
    """
    exec_manager.exit_program()



from tkinter import Menu


def build_popup_menu(editor_instance):
    # Popup menu
    editor_instance.popup_menu = Menu(editor_instance.main_window, tearoff=0)
    editor_instance.popup_menu.add_cascade(label="File", menu=editor_instance.file_menu)
    editor_instance.popup_menu.add_cascade(label="Run", menu=editor_instance.run_menu)



from je_editor.utils.font.font import create_new_font
from je_editor.utils.editor_content.editor_data import editor_data_dict
from tkinter.font import Font


def _config_change(program_run_result_textarea, shell_run_result_textarea, new_font):
    """
    :param program_run_result_textarea: textarea who change font
    :param shell_run_result_textarea: text area who change font
    :param new_font: which font chose
    """
    program_run_result_textarea.configure(font=new_font)
    shell_run_result_textarea.configure(font=new_font)


def change_font(program_run_result_textarea, shell_run_result_textarea, tkinter_font: str):
    """
    :param program_run_result_textarea: which textarea change font
    :param shell_run_result_textarea: which textarea change font
    :param tkinter_font: which font chose
    """
    current_font_size = Font(font=program_run_result_textarea["font"]).actual()["size"]
    new_font = create_new_font(tkinter_font, font_size=current_font_size)
    _config_change(program_run_result_textarea, shell_run_result_textarea, new_font)
    editor_data_dict["font"] = tkinter_font


def change_font_size(program_run_result_textarea, shell_run_result_textarea, font_size: int):
    """
    :param program_run_result_textarea: which  textarea change font
    :param shell_run_result_textarea: which  textarea change font
    :param font_size:which font size choose
    """
    current_font_family = Font(font=program_run_result_textarea["font"]).actual()["family"]
    new_font = create_new_font(current_font_family, font_size=font_size)
    _config_change(program_run_result_textarea, shell_run_result_textarea, new_font)
    editor_data_dict["font_size"] = font_size



from tkinter import Menu

from je_editor.ui.ui_event.change_font.change_font import change_font, change_font_size
from je_editor.ui.ui_event.encoding.set_encoding import set_encoding
from je_editor.ui.ui_event.language.set_language import set_language
from je_editor.utils.encoding.encoding_data_module import encoding_list
from je_editor.utils.font.font import get_font
from je_editor.utils.language.language_data_module import language_list


def build_file_menu(editor_instance):
    # File menu
    editor_instance.file_menu = Menu(editor_instance.menu, tearoff=0)
    # Edit setting menu
    editor_instance.editor_setting_menu = Menu(editor_instance.menu, tearoff=0)
    editor_instance.file_menu.add_command(label="Save File", command=editor_instance.ui_save_file_to_open)
    editor_instance.file_menu.add_command(label="Open File", command=editor_instance.ui_open_file_to_read)
    # Encoding menu
    editor_instance.encoding_menu = Menu(editor_instance.editor_setting_menu, tearoff=0)
    # Language menu
    editor_instance.language_menu = Menu(editor_instance.editor_setting_menu, tearoff=0)
    # Text menu
    editor_instance.text_menu = Menu(editor_instance.editor_setting_menu, tearoff=0)
    editor_instance.text_font_sub_menu = Menu(editor_instance.text_menu, tearoff=0)
    editor_instance.text_size_sub_menu = Menu(editor_instance.text_menu, tearoff=0)
    editor_instance.text_menu.add_cascade(label="Font", menu=editor_instance.text_font_sub_menu)
    editor_instance.text_menu.add_cascade(label="Font Size", menu=editor_instance.text_size_sub_menu)
    editor_instance.font_tuple = get_font(editor_instance.main_window)
    # Encoding menu
    for i in range(len(encoding_list)):
        editor_instance.encoding_menu.add_command(
            label=str(encoding_list[i]),
            command=lambda choose_encoding=encoding_list[i]: set_encoding(editor_instance.exec_manager, choose_encoding)
        )
    # Font menu
    for i in range(len(editor_instance.font_tuple)):
        editor_instance.text_font_sub_menu.add_command(
            label=str(editor_instance.font_tuple[i]),
            command=lambda choose_font=editor_instance.font_tuple[i]:
            change_font(editor_instance.code_editor_textarea, editor_instance.program_run_result_textarea, choose_font)
        )
    # Text size menu
    for i in range(12, 36, 2):
        editor_instance.text_size_sub_menu.add_command(
            label=str(i),
            command=lambda font_size=i: change_font_size(editor_instance.code_editor_textarea,
                                                         editor_instance.program_run_result_textarea,
                                                         font_size)
        )
    # Language menu
    for i in range(len(language_list)):
        editor_instance.language_menu.add_command(
            label=str(language_list[i]),
            command=lambda choose_language=language_list[i]: set_language(editor_instance.exec_manager, choose_language)
        )
    editor_instance.editor_setting_menu.add_cascade(label="Text", menu=editor_instance.text_menu)
    editor_instance.editor_setting_menu.add_cascade(label="Encoding", menu=editor_instance.encoding_menu)
    editor_instance.editor_setting_menu.add_cascade(label="Language", menu=editor_instance.language_menu)
    editor_instance.file_menu.add_separator()
    editor_instance.file_menu.add_cascade(label="Editor Setting", menu=editor_instance.editor_setting_menu)



from je_editor.utils.exception.exceptions import *
# exec code
# editor content
# file
from je_editor.utils.file.open.open_file import open_file
from je_editor.utils.file.save.save_file import save_file
# start editor
from je_editor.ui.editor_main_ui.tkinter_editor import start_editor
from je_editor.ui.editor_main_ui.tkinter_editor import EditorMain



import os
from pathlib import Path
from threading import Lock
from tkinter import filedialog

from je_editor.utils.exception.exceptions import JEditorOpenFileException


def read_file(file):
    """
    :param file: the file we want to read it's whole file path
    :return: read's file and file content
    try
        lock thread
        find file is exist ? and is file ?
        if both is true
            try to open it and read
            return file and content
    finally
        release lock
    """
    lock = Lock()
    try:
        lock.acquire()
        if file != "" and file is not None:
            file_path = Path(file)
            if file_path.exists() and file_path.is_file():
                with open(file, "r+") as open_read_file:
                    return [file, open_read_file.read()]
    except JEditorOpenFileException:
        raise JEditorOpenFileException
    finally:
        lock.release()


def open_file():
    """
    :return: read's file and file content or ""
    open tkinter ask open file dialog
    if not choose
        len(file) = 0 or ""
        :return ""
    """
    cwd = os.getcwd()
    file = filedialog.askopenfilename(title="Open File",
                                      initialdir=cwd,
                                      defaultextension="*.*",
                                      filetypes=(("all files", "*.*"), ("je editor files", "*.jee")))
    if len(file) == 0:
        return ""
    return read_file(file)
